;;;; remote-mailbox-test.scm

;; Issues
;;
;; - should run multiple nodes; N senders M recievers

(import test)

(import (only (chicken format) format))
(include-relative "test-gloss.incl")

(import scheme
  (chicken base)
  (chicken process-context)
  (chicken process)
  (srfi 1)
  (srfi 18)
  miscmacros
  exn-condition
  mailbox
  #;remote-mailbox-adapter
  remote-mailbox-client
  remote-mailbox-server
  remote-mailbox-service)

(define-constant NUM-MSG 5)
(define-constant RCV-NAM "Wong Foo")
(define MB-NAM 'wong-foo)

;sender (client) can talk to many servers
(define (sender)
  (glossf "* Sending ~A messages, then quit" NUM-MSG)
  (let ((rmb (make-remote-mailbox MB-NAM)))
    (define (sendit msg)
      ;(glossf "Sending |~S| to ~S" msg MB-NAM)
      (remote-mailbox-send! rmb msg) )
    ;(glossf "remote-mailboxes: ~S" (remote-mailboxes))
    (test-assert (list? (remote-mailboxes)))
    (test 1 (length (remote-mailboxes)))
    (dotimes (n NUM-MSG)
      ;a message can be any object
      (let ((msg `(message (recipient . ,RCV-NAM) (id . ,n))))
        (sendit msg) ) )
    (sendit '()))
    #; ;???
    (close-output-port (serializer-output)) )

(define (remote-mailbox-service-server)
  (receive (s t) (remote-mailbox-service-info) s) )

(define (remote-mailbox-service-mailboxes)
  (server-local-mailboxes (remote-mailbox-service-server)) )

;receiver (server) can talk to many clients
(define (receiver)
  (let ((mb (remote-mailbox-service MB-NAM #t #;RCV-NAM)))
    ;(glossf "server-local-mailboxes: ~S" (remote-mailbox-service-mailboxes))
    (test-assert (list? (remote-mailbox-service-mailboxes)))
    (test 1 (length (remote-mailbox-service-mailboxes)))
    (handle-exceptions ex
      (begin (write-exn-condition ex) (newline))
      ;
      (glossf "* Receiving messages until quit...")
      (let receive-loop ((n 0))
        (let ((msg (mailbox-receive! mb)))
          ;(glossf "Received |~S| from ~S" msg MB-NAM)
          (test-assert (list? msg))
          (unless (null? msg)
            (test-assert (eq? 'message (car msg)))
            (test `(id . ,n) (assq 'id (cdr msg)))
            (receive-loop (add1 n)) ) ) )
      (glossf "stopping mailbox server...")
      (remote-mailbox-service-stop!) ) ) )

;;;

(define operation
  (let ((args (command-line-arguments)))
    (and (pair? args)
         (string->symbol (last args))) ) )

(if operation
  ;then know what to do
  (case operation
    ((sender)
      (sender) )
    ((receiver)
      (receiver) )
    (else
      (error 'remote-mailbox-test "unrecognized operation: " operation) ) )
  ;else test w/ this binary
  (let ((cmd (first (argv))))
    ;the "server" MUST be started before the client!
    (glossf "Running Receiver: ~S" cmd)
    ;csi args are ignored by this as a compiled binary - last arg is ours
    (process-run cmd `("-n" "-s" ,(program-name) "receiver"))
    ;wait for receiver to startup
    (sleep 1)
    ;then start the sender
    (glossf "Running Sender: ~S" cmd)
    ;this process not continuing
    ;(process-execute cmd `("-n" "-s" ,(program-name) "sender"))
    (sender) ) )
